using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Application = System.Windows.Forms.Application;
using Form = System.Windows.Forms.Form;
using Model;
using View;
using Blaze_Brigade;
using System.Diagnostics;

namespace Controller

{
    /// <summary>
    /// Main Controller for game
    /// </summary>
    public class Game : Microsoft.Xna.Framework.Game
    {

        #region Variables

        static Game instance;

        public static Game Instance
        {
            get { return instance; }
        }

        GameMenuState currentGameState = GameMenuState.MainMenu; // game starts in main menu screen
        MainMenu mMenu; // main menu variable
        View.HowToPlay tut; // instruction screen variable
        View.HowToPlay2 tut2; // instruction 2 screen variable
        View.HowToPlay3 tut3; // instruction 3 screen variable
        Graph graph;
        Player player1;
        Player player2;

        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;
        Texture2D backGround, moveableNode, attackableNode;
        private SpriteFont font; // custom font
        private SpriteFont largeFont; // custom font 2

        #endregion

        // constructor for game
        public Game()
        {
            instance = this;

            graphics = new GraphicsDeviceManager(this);
            graphics.PreferredBackBufferWidth = GameState.SCREEN_HEIGHT;
            graphics.PreferredBackBufferHeight = GameState.SCREEN_WIDTH;
            Content.RootDirectory = "Content";

            //instantiate the game windows
            mMenu = new MainMenu();
            tut = new HowToPlay();
            tut2 = new HowToPlay2();
            tut3 = new HowToPlay3();
        }

        //initializes game
        protected override void Initialize()
        {
            Form MyGameForm = (Form)Form.FromHandle(Window.Handle); // creates handle
            MyGameForm.Opacity = 0; // hides the game window until it's set to visible
            base.Initialize();
        }

        //loads required textures and values for starting game
        protected override void LoadContent()
        {
            // creates a new SpriteBatch, which can be used to draw textures
            spriteBatch = new SpriteBatch(GraphicsDevice);

            initializeGame();

            backGround = Content.Load<Texture2D>("Game_Map"); // load background
            moveableNode = Content.Load<Texture2D>("moveableNode");
            attackableNode = Content.Load<Texture2D>("attackableNode");

            font = Content.Load<SpriteFont>("PixelFont"); //loads font PixelFont
            largeFont = Content.Load<SpriteFont>("PixelFontLarge"); //loads font PixelFont

            graphics.PreferredBackBufferWidth = GameState.SCREEN_WIDTH; // width of screen
            graphics.PreferredBackBufferHeight = GameState.SCREEN_HEIGHT; //height of screen

            IsMouseVisible = true; // sets mouse visibility to true
            graphics.ApplyChanges(); // load images
        }

        //initializes players and units
        private void initializeGame()
        {
            graph = new Graph(50, 32);
            player1 = new Player();
            player2 = new Player();

            // load character sprite and set position


            player1.addUnit(getNewUnit(UnitType.Warrior, new Vector2(4 * 32f, 6 * 32f), 1));
            player2.addUnit(getNewUnit(UnitType.Warrior, new Vector2(8 * 32f, 6 * 32f), 2));

            GameState.currentPlayer = (player1);
            GameState.enemyPlayer = (player2);
            setObstacles(graph);

        }

        // Updates game in real time - 60fps
        protected override void Update(GameTime gameTime)
        {
            #region Exiting Game
            // allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == Microsoft.Xna.Framework.Input.ButtonState.Pressed)
                this.Exit();

            if (this.IsActive)
            {
                // call mouse handler if game window is active
                MouseHandler.updateMouse(graph);
            }
            #endregion

            #region Game States
            MouseState mouse = Mouse.GetState();
            //Lists possible game states
            switch (currentGameState)
            {
                case GameMenuState.MainMenu: // if mouse is clicked... call method
                    mMenu.Show(); // show main menu
                    if (mMenu.start == true) //if New game is selected
                    {
                        mMenu.Close(); //close Main Menu screen
                        tut.Close(); //close How To Play Menu
                        Form GameForm = (Form)Form.FromHandle(Window.Handle);
                        GameForm.Opacity = 100; // make screen show
                        currentGameState = GameMenuState.Playing; //set game state to Playing
                        break;
                    }
                    if (mMenu.getInstruct() == true) //if How to Play is selected
                    {
                        mMenu.Hide(); //hide visibility of menu window
                        currentGameState = GameMenuState.HowToPlay; //change game state to How To Play
                        break;
                    }
                    if (mMenu.quit == true) // if quit clicked
                    {
                        this.Exit();
                        break;
                    }
                    break;

                case GameMenuState.HowToPlay: // if true.. load new image...
                    tut.Show(); //set visibility of how to play window to true
                    mMenu.setInstructFalse();
                    if (tut.getNext() == true) //if How to Play is selected
                    {
                        tut.Hide(); //hide visibility of menu window
                        currentGameState = GameMenuState.HowToPlay2; //change game state to How To Play
                        break;
                    }
                    if (tut.getQuit() == true) // if quit clicked
                    {
                        tut.setQuitFalse();
                        tut.Hide(); //hide 
                        currentGameState = GameMenuState.MainMenu; //changes state back to main menu
                        break;
                    }
                    break;

                case GameMenuState.HowToPlay2: // if true.. load new image...
                    tut2.Show(); //set visibility of how to play window to true
                    tut.setNextFalse();
                    if (tut2.getNext() == true) //if How to Play is selected
                    {
                        tut2.Hide(); //hide visibility of menu window
                        currentGameState = GameMenuState.HowToPlay3; //change game state to How To Play
                        break;
                    }
                    if (tut2.getQuit() == true) // if quit clicked
                    {
                        tut2.setQuitFalse();
                        tut2.Hide(); //hide 
                        currentGameState = GameMenuState.HowToPlay; //changes state back to main menu
                        break;
                    }
                    break;

                case GameMenuState.HowToPlay3: // if true.. load new image...
                    tut3.Show(); //set visibility of how to play window to true
                    tut2.setNextFalse();
                    if (tut3.getQuit() == true) // if quit clicked
                    {
                        tut3.setQuitFalse();
                        tut3.Hide(); //hide 
                        currentGameState = GameMenuState.HowToPlay2; //changes state back to main menu
                        break;
                    }
                    break;

                case GameMenuState.Playing: // if true.. load new image...
                    backGround = Content.Load<Texture2D>("Game_Map"); // load background
                    if (GameFunction.isTurnOver())
                    {
                        Player tempPlayer = GameState.currentPlayer;
                        GameState.currentPlayer = (GameState.enemyPlayer);
                        GameState.enemyPlayer = (tempPlayer);
                        GameFunction.startTurn(GameState.currentPlayer);
                    }
                    break;
            }
            #endregion

            base.Update(gameTime); // repeatedly calls update
        }

        //draws the game as it updates
        protected override void Draw(GameTime gameTime)
        {
            spriteBatch.Begin(SpriteSortMode.BackToFront, BlendState.AlphaBlend, null, null, null, null);   // begin spriteBatch

            //only draw objects relevent to current game state
            switch (currentGameState)
            {
                case GameMenuState.Playing: // while in game

                    #region Player 1 Units
                    // draws units for player 1
                    for (int i = 0; i < player1.getNumOfUnits(); i++)
                    {
                        Unit unit = player1.getUnits().ElementAt(i); //gets unit at i
                        spriteBatch.Draw(unit.getSpriteImage(), unit.PixelCoordinates,
                            unit.getCurrentFrame(), Color.White, 0, Vector2.Zero, 1f, SpriteEffects.None, 0.8f);
                    }
                    #endregion

                    #region Player 2 Units
                    // draws units for player 2
                    for (int i = 0; i < player2.getNumOfUnits(); i++)
                    {
                        Unit unit = player2.getUnits().ElementAt(i);
                        spriteBatch.Draw(unit.getSpriteImage(), unit.PixelCoordinates,
                            unit.getCurrentFrame(), Color.White, 0, Vector2.Zero, 1f, SpriteEffects.None, 0.8f);
                    }
                    #endregion

                    #region When Unit is selected
                    //if unit is currently clicked on
                    if (GameState.playableUnitSelected)
                    {
                        Unit unit = GameState.selectedUnit;
                        if (!GameState.isAnimating)
                        {
                            #region Highlight nodes
                            if (GameState.beforeMove) // if unit has yet to move, display the overall move and attack range of unit
                            {
                                
                                // Highlight movable nodes in blue
                                foreach (Node move in GameState.moveableNodes)
                                {
                                    spriteBatch.Draw(moveableNode, move.getPosition(), null, Color.White * 0.2f, 0, Vector2.Zero, 1f, SpriteEffects.None, 0.9f);
                                }

                                // Highlight attackable nodes in red
                                LinkedList<Node> attackableNodes = GameFunction.getAttackableNodes(graph, unit);
                                foreach (Node attack in attackableNodes)
                                {
                                    if (!GameState.moveableNodes.Contains(attack))
                                    {
                                        spriteBatch.Draw(attackableNode, attack.getPosition(), null, Color.White * 0.2f, 0, Vector2.Zero, 1f, SpriteEffects.None, 0.9f);
                                    }
                                }
                                
                            }
                            else // else if unit has already moved, only display the attack range
                            {
                                LinkedList<Node> attackableNodes = GameFunction.getAttackRangeAfterMoving(graph, unit);
                                foreach (Node attack in attackableNodes)
                                {
                                    spriteBatch.Draw(attackableNode, attack.getPosition(), null, Color.White * 0.2f, 0, Vector2.Zero, 1f, SpriteEffects.None, 0.9f);
                                }
                            }
                            #endregion

                            #region Drop Down menu
                            if (GameState.dropDownMenuOpen) // if dropDowMenu should be opened, draw dropDownMenu
                            {
                                unit.setMenuButtonCoordinates(unit.PixelCoordinates);
                                foreach (MenuButton button in unit.getMenuButtons())
                                {
                                    if (button.getActive())
                                    {
                                        spriteBatch.Draw(button.getImage(), button.getPixelCoordinates(), null, Color.White, 0, Vector2.Zero, 1f, SpriteEffects.None, 0f);
                                    }
                                }
                            }
                            #endregion
                        }

                        #region Character Info Screen player1

                        //if player 1, prints info screen for player 1
                        if (GameState.currentPlayer == player1)
                        {
                            Vector2 statLocation = new Vector2(180, 533); //starting location for first stat
                            Vector2 increment = new Vector2(0, 11.5f); //increment downwards for each stat
                            Vector2 infoLocation = new Vector2(20, 513);

                            for (int k = 0; k < 7; k++) //for 6 stats - str, int, skill, speed, def, res
                            {
                                spriteBatch.DrawString(font, unit.getStats(k).ToString(), statLocation, Color.White, 0, Vector2.Zero, 1f, SpriteEffects.None, 0.6f); //draws each stat
                                statLocation = statLocation + increment; //increment downwards
                            }
                            spriteBatch.DrawString(largeFont, unit.Hp.ToString(), new Vector2(249, 512), Color.White, 0, Vector2.Zero, 1f, SpriteEffects.None, 0.6f); //draws each stat

                            spriteBatch.Draw(unit.getCharInfo(), infoLocation, null, Color.White, 0, Vector2.Zero, 1f, SpriteEffects.None, 0.7f); //draw charInfoBackground texture
                        }
                        //else, info screen for player 2
                        else
                        {
                            Vector2 statLocation = new Vector2(795, 533); //starting location for first stat
                            Vector2 increment = new Vector2(0, 11.5f); //increment downwards for each stat
                            Vector2 infoLocation = new Vector2(635, 513);

                            for (int k = 0; k < 7; k++) //for 6 stats - str, int, skill, speed, def, res
                            {
                                spriteBatch.DrawString(font, unit.getStats(k).ToString(), statLocation, Color.White, 0, Vector2.Zero, 1f, SpriteEffects.None, 0.6f); //draws each stat
                                statLocation = statLocation + increment; //increment downwards
                            }
                            spriteBatch.DrawString(largeFont, unit.Hp.ToString(), new Vector2(864, 512), Color.White, 0, Vector2.Zero, 1f, SpriteEffects.None, 0.6f); //draws each stat
                            spriteBatch.Draw(unit.getCharInfo(), infoLocation, null, Color.White, 0, Vector2.Zero, 1f, SpriteEffects.None, 0.7f); //draw charInfoBackground texture
                        }
                        #endregion

                    }
                    #endregion
     
                    spriteBatch.Draw(backGround, Vector2.Zero, null, Color.White, 0, Vector2.Zero, 1f, SpriteEffects.None, 1);

                    break;
            }

            spriteBatch.End();  // end spriteBatch
            base.Draw(gameTime);    // repeatedly calls draw
        }

        //method to return a unit: takes in unit type (warrior/mage/ranger), unit position, and which player's unit (since different model/colors)
        private Unit getNewUnit(UnitType unitType, Vector2 unitPosition, int player)
        {
            MenuButton attackButton = new MenuButton(MenuButtonType.Attack, unitPosition, Content.Load<Texture2D>("attack"));
            MenuButton moveButton = new MenuButton(MenuButtonType.Move, unitPosition, Content.Load<Texture2D>("move"));
            MenuButton itemButton = new MenuButton(MenuButtonType.Items, unitPosition, Content.Load<Texture2D>("items")); ;
            MenuButton waitButton = new MenuButton(MenuButtonType.Wait, unitPosition, Content.Load<Texture2D>("wait")); ;

            if (player == 1)
            {
                if (unitType == UnitType.Warrior)
                {
                    Unit unit = new Warrior(Content.Load<Texture2D>("warrior"), attackButton, moveButton,
                        itemButton, waitButton, Content.Load<Texture2D>("warrior_stats"), unitPosition, 1);
                    graph.getNode(unitPosition).unitOnNode = (unit);
                    return unit;
                }
            }
            else
            {
                if (unitType == UnitType.Warrior)
                {
                    Unit unit = new Warrior(Content.Load<Texture2D>("2warrior"), attackButton, moveButton,
                        itemButton, waitButton, Content.Load<Texture2D>("2warrior_stats"), unitPosition, 2);
                    graph.getNode(unitPosition).unitOnNode = (unit);
                    return unit;
                }
            }

            return null;
        }

        // manually sets obstacle nodes for graph
        private void setObstacles(Graph graph)
        {
            for (int y = 5; y <= 14; y++)
            {
                graph.getNode(2, y).isObstacle=(true);
            }
            graph.getNode(3, 5).isObstacle=(true);
            graph.getNode(3, 13).isObstacle=(true);
            graph.getNode(4, 3).isObstacle=(true);
            graph.getNode(4, 4).isObstacle=(true);
            graph.getNode(4, 13).isObstacle=(true);
            graph.getNode(4, 14).isObstacle=(true);
            graph.getNode(5, 2).isObstacle=(true);
            graph.getNode(5, 15).isObstacle=(true);
            graph.getNode(6, 2).isObstacle=(true);
            graph.getNode(6, 14).isObstacle=(true);
            graph.getNode(7, 2).isObstacle=(true);
            graph.getNode(7, 3).isObstacle=(true);
            graph.getNode(7, 4).isObstacle=(true);
            graph.getNode(7, 14).isObstacle=(true);
            graph.getNode(8, 4).isObstacle=(true);
            for (int y = 14; y <= 17; y++)
            {
                graph.getNode(8, y).isObstacle=(true);
            }
            graph.getNode(9, 4).isObstacle=(true);
            for (int y = 17; y <= 20; y++)
            {
                graph.getNode(9, y).isObstacle=(true);
            }
            graph.getNode(10, 4).isObstacle=(true);
            graph.getNode(10, 20).isObstacle=(true);
            graph.getNode(10, 21).isObstacle=(true);
            graph.getNode(10, 22).isObstacle=(true);
            graph.getNode(11, 4).isObstacle=(true);
            graph.getNode(11, 14).isObstacle=(true);
            graph.getNode(11, 22).isObstacle=(true);
            graph.getNode(12, 3).isObstacle=(true);
            for (int y = 22; y <= 25; y++)
            {
                graph.getNode(12, y).isObstacle=(true);
            }
            graph.getNode(13, 4).isObstacle=(true);
            graph.getNode(12, 25).isObstacle=(true);
            graph.getNode(14, 4).isObstacle=(true);
            graph.getNode(14, 25).isObstacle=(true);
            graph.getNode(15, 4).isObstacle=(true);
            graph.getNode(15, 25).isObstacle=(true);
            graph.getNode(15, 26).isObstacle=(true);
            graph.getNode(15, 27).isObstacle=(true);
            graph.getNode(16, 3).isObstacle=(true);
            graph.getNode(16, 27).isObstacle=(true);
            graph.getNode(17, 2).isObstacle=(true);
            graph.getNode(17, 3).isObstacle=(true);
            graph.getNode(17, 27).isObstacle=(true);
            graph.getNode(18, 1).isObstacle=(true);
            graph.getNode(18, 27).isObstacle=(true);
            graph.getNode(19, 1).isObstacle=(true);
            graph.getNode(19, 27).isObstacle=(true);
            graph.getNode(20, 2).isObstacle=(true);
            for (int y = 24; y <= 27; y++)
            {
                graph.getNode(20, y).isObstacle=(true);
            }
            graph.getNode(21, 2).isObstacle=(true);
            graph.getNode(21, 21).isObstacle=(true);
            graph.getNode(21, 22).isObstacle=(true);
            graph.getNode(21, 23).isObstacle=(true);
            graph.getNode(22, 1).isObstacle=(true);
            graph.getNode(22, 2).isObstacle=(true);
            graph.getNode(22, 20).isObstacle=(true);
            graph.getNode(23, 0).isObstacle=(true);
            graph.getNode(23, 20).isObstacle=(true);
            graph.getNode(24, 0).isObstacle=(true);
            graph.getNode(24, 20).isObstacle=(true);
            graph.getNode(25, 1).isObstacle=(true);
            graph.getNode(25, 20).isObstacle=(true);
            graph.getNode(26, 1).isObstacle=(true);
            graph.getNode(26, 21).isObstacle=(true);
            graph.getNode(27, 1).isObstacle=(true);
            graph.getNode(27, 22).isObstacle=(true);
            graph.getNode(28, 0).isObstacle=(true);
            for (int y = 22; y <= 27; y++)
            {
                graph.getNode(28, y).isObstacle=(true);
            }
            graph.getNode(29, 1).isObstacle=(true);
            graph.getNode(29, 2).isObstacle=(true);
            graph.getNode(29, 3).isObstacle=(true);
            graph.getNode(29, 27).isObstacle=(true);
            graph.getNode(30, 3).isObstacle=(true);
            graph.getNode(30, 27).isObstacle=(true);
            graph.getNode(31, 2).isObstacle=(true);
            graph.getNode(31, 3).isObstacle=(true);
            graph.getNode(31, 27).isObstacle=(true);
            graph.getNode(32, 27).isObstacle=(true);
            graph.getNode(33, 2).isObstacle=(true);
            graph.getNode(33, 3).isObstacle=(true);
            graph.getNode(33, 26).isObstacle=(true);
            graph.getNode(34, 3).isObstacle=(true);
            graph.getNode(34, 25).isObstacle=(true);
            graph.getNode(35, 3).isObstacle=(true);
            graph.getNode(35, 25).isObstacle=(true);
            graph.getNode(36, 2).isObstacle=(true);
            graph.getNode(36, 25).isObstacle=(true);
            graph.getNode(37, 2).isObstacle=(true);
            graph.getNode(37, 25).isObstacle=(true);
            graph.getNode(38, 2).isObstacle=(true);
            for (int y = 9; y <= 14; y++)
            {
                graph.getNode(38, y).isObstacle=(true);
            }
            graph.getNode(38, 26).isObstacle=(true);
            graph.getNode(39, 2).isObstacle=(true);
            graph.getNode(39, 8).isObstacle=(true);
            graph.getNode(39, 14).isObstacle=(true);
            graph.getNode(40, 3).isObstacle=(true);
            graph.getNode(40, 7).isObstacle=(true);
            graph.getNode(40, 14).isObstacle=(true);
            graph.getNode(40, 26).isObstacle=(true);
            graph.getNode(41, 3).isObstacle=(true);
            graph.getNode(41, 7).isObstacle=(true);
            for (int y = 14; y <= 17; y++)
            {
                graph.getNode(41, y).isObstacle=(true);
            }
            graph.getNode(41, 27).isObstacle=(true);
            graph.getNode(42, 4).isObstacle=(true);
            graph.getNode(42, 7).isObstacle=(true);
            graph.getNode(42, 17).isObstacle=(true);
            graph.getNode(42, 27).isObstacle=(true);
            graph.getNode(43, 4).isObstacle=(true);
            graph.getNode(43, 7).isObstacle=(true);
            graph.getNode(43, 17).isObstacle=(true);
            graph.getNode(43, 27).isObstacle=(true);
            graph.getNode(44, 4).isObstacle=(true);
            graph.getNode(44, 7).isObstacle=(true);
            for (int y = 17; y <= 23; y++)
            {
                graph.getNode(44, y).isObstacle=(true);
            }
            graph.getNode(44, 27).isObstacle=(true);
            graph.getNode(45, 4).isObstacle=(true);
            graph.getNode(45, 5).isObstacle=(true);
            graph.getNode(45, 24).isObstacle=(true);
            graph.getNode(45, 25).isObstacle=(true);
            graph.getNode(45, 27).isObstacle=(true);
            graph.getNode(46, 5).isObstacle=(true);
            graph.getNode(46, 7).isObstacle=(true);
            graph.getNode(46, 25).isObstacle=(true);
            graph.getNode(46, 26).isObstacle=(true);
            graph.getNode(47, 5).isObstacle=(true);
            graph.getNode(47, 6).isObstacle=(true);
            graph.getNode(47, 7).isObstacle=(true);
        }
    }
}
