﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Model;
using View;

namespace Controller
{
    // This class holds useable functions in the scope of the entire gameplay
    static class GameFunction
    {
        // might not need; check if enemyUnitsInRange returns an empty list
        public static bool isAnEnemyUnitInRange(Unit unit)
        {
            // TODO
            return false;
        }

        // sets all units to "not moved" upon start of a turn
        public static void startTurn(Player player)
        {
            foreach (Unit unit in player.getUnits())
            {
                MenuButton[] menuButtons = unit.getMenuButtons();
                for (int i = 0; i < menuButtons.Count(); i++)
                {
                    menuButtons[i].setActive(true);
                }
            }
        }

        // returns all enemy units in range of the specified unit
        public static LinkedList<Unit> enemyUnitsInRange(Unit unit)
        {
            // TODO
            //int range = unit.getEquippedWeapon().getRange();    // use this to determine all hitable squares
            return null;
        }

        // checks if specified unit can perform actions
        public static bool hasUnitFinishedActions(Unit unit)
        {
            if (!unit.isButtonActive(MenuButtonType.Attack))
            {
                return true;
            }
            if (!unit.isButtonActive(MenuButtonType.Wait))
            {
                return true;
            }
            return false;
        }

        public static bool isTurnOver()
        {
            foreach (Unit unit in GameState.currentPlayer.getUnits())
            {
                if (!hasUnitFinishedActions(unit))
                {
                    return false;
                }
            }
            return true;
        }

        // checks lose/win conditions to determine if game is over
        public static bool isGameOver()
        {
            // TODO
            return false;
        }

        // checks if the node is attackable based on the unit's class and position
        // I THINK THIS WOULD WORK MUCH SIMPLER BASED ON EQUIPPED WEAPON NOT UNIT TYPE!!
        // IF logic < equippedweapon.range or IF logic < weapon with highest range.
        private static bool isNodeAttackable(UnitType unitType, int unitX, int unitY, Node attackNode)
        {
            int attackX = attackNode.getPositionX();
            int attackY = attackNode.getPositionY();

            if (attackNode.isObstacle)
            {
                return false;
            }

            // return warrior attackable nodes (one node away)
            if (unitType == UnitType.Warrior &&
                Math.Abs(unitX - attackX) + Math.Abs(unitY - attackY) == 1)
            {
                return true;
            }
            // return archer attackable nodes (two nodes away)
            else if (unitType == UnitType.Archer &&
                Math.Abs(unitX - attackX) + Math.Abs(unitY - attackY) == 2)
            {
                return true;
            }
            // return mage attackable nodes (one node and two nodes away)
            else if (unitType == UnitType.Mage &&
                (Math.Abs(unitX - attackX) + Math.Abs(unitY - attackY) == 1 ||
                    Math.Abs(unitX - attackX) + Math.Abs(unitY - attackY) == 2))
            {
                return true;
            }

            return false;
        }

        // returns a list of nodes that the unit can move onto
        public static LinkedList<Node> setMovableNodes(Graph graph, Unit unit)
        {
            LinkedList<Node> moveableNodes = new LinkedList<Node>();
            Node currentNode = graph.getNode(unit.Position);

            // iterate through all nodes in the graph
            for (int x = 0; x < graph.getWidth(); x++)
            {
                for (int y = 0; y < graph.getHeight(); y++)
                {
                    // if a path exists to that node, add it to list of moveable nodes
                    if (pathFinder(graph, unit, currentNode, graph.getNode(x, y)) != null && !graph.getNode(x, y).isOccupied())
                    {
                        moveableNodes.AddLast(graph.getNode(x, y));
                    }
                }
            }
            GameState.moveableNodes = moveableNodes;
            return moveableNodes;
        }

        // returns a list of nodes that the unit can attack on (ie. used for highlighting attackable nodes)
        public static LinkedList<Node> getAttackableNodes(Graph graph, Unit unit)
        {
            LinkedList<Node> attackableNodes = new LinkedList<Node>();

            // determine attackable nodes for each moveable node
            foreach (Node moveableNode in GameState.moveableNodes)
            {
                // iterate through entire grid to determine attackable nodes for moveable node
                for (int x = 0; x < graph.getWidth(); x++)
                {
                    for (int y = 0; y < graph.getHeight(); y++)
                    {
                        if (isNodeAttackable(unit.getClass(), moveableNode.getPositionX(),
                            moveableNode.getPositionY(), graph.getNode(x, y)))
                        {
                            // if node is attackable, add it to list of attackable nodes
                            attackableNodes.AddLast(graph.getNode(x, y));
                        }
                    }
                }
            }
            return attackableNodes;
        }

        public static LinkedList<Node> getAttackRangeAfterMoving(Graph graph, Unit unit)
        {
            LinkedList<Node> attackableNodes = new LinkedList<Node>();
            Tuple<int, int> currentPosition = unit.Position;
            for (int x = 0; x < graph.getWidth(); x++)
            {
                for (int y = 0; y < graph.getHeight(); y++)
                {
                    if (isNodeAttackable(unit.getClass(), currentPosition.Item1, currentPosition.Item2, graph.getNode(x, y)))
                    {
                        // if node is attackable, add it to list of attackable nodes
                        attackableNodes.AddLast(graph.getNode(x, y));
                    }
                }
            }
            return attackableNodes;
        }

        // returns list of nodes representing the path from start node to end node; if no path is valid, return null
        public static LinkedList<Node> pathFinder(Graph graph, Unit unit, Node start, Node end)
        {
            // TODO: (OPTIONAL, IGNORE FOR NOW) incorporate movability of nodes in the calculation

            if (end.isObstacle || end.isOccupied())
            {
                return null;
            }

            // uses a variation of breadth-first search
            Queue<LinkedList<Node>> paths = new Queue<LinkedList<Node>>();  // queue of paths

            // sets up queue of paths
            LinkedList<Node> startPath = new LinkedList<Node>();
            startPath.AddFirst(start);      // adds the starting node to starting path
            paths.Enqueue(startPath);

            while (paths.Count != 0)    // while queue isn't empty
            {
                LinkedList<Node> path = paths.Dequeue();    // get the first path from the queue
                Node lastNode = path.Last();                // get the last node from the path
                if (lastNode == end)                        // if last node is desired end node, return path
                {
                    return path;
                }
                if (path.Count <= unit.getMovability())     // only consider paths within the unit's movability
                {
                    // add paths with adjacent nodes of last node to queue
                    foreach (Node adjacentNode in getAdjacentNodes(graph, lastNode))
                    {
                        if (!path.Contains(adjacentNode))   // if adjacent node doesn't already exist in the path
                        {
                            LinkedList<Node> newPath = new LinkedList<Node>();
                            // copies path to new path
                            foreach (Node node in path)
                            {
                                newPath.AddLast(node);
                            }
                            // add adjacent node to new path that will be enqueued
                            newPath.AddLast(adjacentNode);
                            paths.Enqueue(newPath);
                        }
                    }
                }
            }
            return null;
        }

        // returns movable adjacent nodes; used for BFS path finding
        private static LinkedList<Node> getAdjacentNodes(Graph graph, Node node)
        {
            LinkedList<Node> adjacentNodes = new LinkedList<Node>();
            Node upNode = graph.getNode(node.getPositionX(), node.getPositionY() + 1);
            Node downNode = graph.getNode(node.getPositionX(), node.getPositionY() - 1);
            Node rightNode = graph.getNode(node.getPositionX() + 1, node.getPositionY());
            Node leftNode = graph.getNode(node.getPositionX() - 1, node.getPositionY());

            // up node
            if (!upNode.isOccupied() && !upNode.isObstacle)
            {
                adjacentNodes.AddLast(upNode);
            }

            // down node
            if (!downNode.isOccupied() && !downNode.isObstacle)
            {
                adjacentNodes.AddLast(downNode);
            }

            // left node
            if (!leftNode.isOccupied() && !leftNode.isObstacle)
            {
                adjacentNodes.AddLast(leftNode);
            }

            // right node
            if (!rightNode.isOccupied() && !rightNode.isObstacle)
            {
                adjacentNodes.AddLast(rightNode);
            }

            return adjacentNodes;
        }
    }
}

