﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;

namespace Model
{
    class Graph
    {
        private int numberOfNodes;  // total number of nodes in the graph
        private int width;          // width of the graph (by number of nodes)
        private int height;         // height of the graph (by number of nodes)
        private Node[,] nodes;      // list of nodes in the graph

        public Graph(int x, int y)
        {
            this.width = x;
            this.height = y;
            numberOfNodes = x * y;

            // initialize nodes inside the graph
            nodes = new Node[x, y];
            for (int i = 0; i < x; i++)
            {
                for (int j = 0; j < y; j++)
                {
                    nodes[i, j] = new Node(i, j);
                }
            }
        }

        public int getNumberOfNodes()
        {
            return numberOfNodes;
        }

        public Node getNode(int x, int y)
        {
            return nodes[x, y];
        }

        public Node getNode(Vector2 pixelCoordinates)
        {
            int x = (int) Math.Floor(pixelCoordinates.X / 32);
            int y = (int) Math.Floor(pixelCoordinates.Y / 32);
            return nodes[x, y];
        }

        public Node getNode(Tuple<int, int> position)
        {
            return nodes[position.Item1, position.Item2];
        }

        public void setNode(Node node, int x, int y)
        {
            nodes[x, y] = node;
        }

        public int getWidth()
        {
            return width;
        }

        public int getHeight()
        {
            return height;
        }

        public void setAllNodesUnmarked()
        {
            for (int i = 0; i < width; i++)
            {
                for (int j = 0; j < height; j++)
                {
                    nodes[i, j].isMarked = (false);
                }
            }
        }
    }
}
